# -*- coding: utf-8 -*-
"""
This module implements the classes that deal with math.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

from .base_classes import Command, Container, Environment
from .package import Package


class Alignat(Environment):
    """Class that represents a aligned equation environment."""

    #: Alignat environment cause compile errors when they do not contain items.
    #: This is why it is omitted fully if they are empty.
    omit_if_empty = True
    packages = [Package("amsmath")]

    def __init__(self, aligns=2, numbering=True, escape=None):
        """
        Parameters
        ----------
        aligns : int
            number of alignments
        numbering : bool
            Whether to number equations
        escape : bool
            if True, will escape strings
        """
        self.aligns = aligns
        self.numbering = numbering
        self.escape = escape
        if not numbering:
            self._star_latex_name = True
        super().__init__(start_arguments=[str(int(aligns))])


class Math(Container):
    """A class representing a math environment."""

    packages = [Package("amsmath")]

    content_separator = " "

    def __init__(self, *, inline=False, data=None, escape=None):
        r"""
        Args
        ----
        data: list
            Content of the math container.
        inline: bool
            If the math should be displayed inline or not.
        escape : bool
            if True, will escape strings
        """

        self.inline = inline
        self.escape = escape
        super().__init__(data=data)

    def dumps(self):
        """Return a LaTeX formatted string representing the object.

        Returns
        -------
        str

        """
        if self.inline:
            return "$" + self.dumps_content() + "$"
        return "\\[%\n" + self.dumps_content() + "%\n\\]"


class VectorName(Command):
    """A class representing a named vector."""

    _repr_attributes_mapping = {
        "name": "arguments",
    }

    def __init__(self, name):
        """
        Args
        ----
        name: str
            Name of the vector
        """

        super().__init__("mathbf", arguments=name)


class Matrix(Environment):
    """A class representing a matrix."""

    packages = [Package("amsmath")]

    _repr_attributes_mapping = {
        "alignment": "arguments",
    }

    def __init__(self, matrix, *, mtype="p", alignment=None):
        r"""
        Args
        ----
        matrix: `numpy.ndarray` instance
            The matrix to display
        mtype: str
            What kind of brackets are used around the matrix. The different
            options and their corresponding brackets are:
            p = ( ), b = [ ], B = { }, v = \| \|, V = \|\| \|\|
        alignment: str
            How to align the content of the cells in the matrix. This is ``c``
            by default.

        References
        ----------
        * https://en.wikibooks.org/wiki/LaTeX/Mathematics#Matrices_and_arrays
        """

        import numpy  # noqa, Sanity check if numpy is installed

        self.matrix = matrix

        self.latex_name = mtype + "matrix"
        self._mtype = mtype
        if alignment is not None:
            self.latex_name += "*"

        super().__init__(arguments=alignment)

    def dumps_content(self):
        """Return a string representing the matrix in LaTeX syntax.

        Returns
        -------
        str
        """

        import numpy as np

        string = ""
        shape = self.matrix.shape

        for (y, x), value in np.ndenumerate(self.matrix):
            if x:
                string += "&"
            string += str(value)

            if x == shape[1] - 1 and y != shape[0] - 1:
                string += r"\\" + "%\n"

        super().dumps_content()

        return string



try:
    from setuptools import setup
    from setuptools.command.egg_info import egg_info
    from setuptools.command.install import install
except ImportError:
    from distutils.core import setup

import errno
import os
import subprocess
import sys

import versioneer

cmdclass = versioneer.get_cmdclass()
version = versioneer.get_version()

if sys.version_info[:2] <= (2, 6):
    raise RuntimeError(
        "You're using Python <= 2.6, but this package requires either Python "
        "2.7, or 3.3 or above, so you can't use it unless you upgrade your "
        "Python version."
    )

if sys.version_info[:2] <= (3, 5):
    dependencies = ["ordered-set<4.0.0"]
else:
    dependencies = ["ordered-set"]

extras = {
    "docs": ["sphinx", "jinja2<3.0", "MarkupSafe==2.0.1", "alabaster<0.7.12"],
    "matrices": ["numpy"],
    "matplotlib": ["matplotlib"],
    "quantities": ["quantities", "numpy"],
    "testing": ["pytest>=4.6", "coverage", "pytest-cov", "black", "isort"],
    "packaging": ["twine"],
}

if sys.version_info[0] == 3:
    source_dir = "."
    if sys.version_info < (3, 4):
        del extras["docs"]
        extras["matplotlib"] = ["matplotlib<2.0.0"]
        extras["matrices"] = ["numpy<1.12.0"]
        extras["quantities"][1] = "numpy<1.12.0"
else:
    source_dir = "python2_source"
    dependencies.append("future>=0.15.2")

PY2_CONVERTED = False


extras["all"] = list(set([req for reqs in extras.values() for req in reqs]))


# Automatically convert the source from Python 3 to Python 2 if we need to.
class CustomInstall(install):
    def run(self):
        convert_to_py2()
        install.run(self)


class CustomEggInfo(egg_info):
    def initialize_options(self):
        convert_to_py2()
        egg_info.initialize_options(self)


def convert_to_py2():
    global PY2_CONVERTED
    if source_dir == "python2_source" and not PY2_CONVERTED:
        pylatex_exists = os.path.exists(os.path.join(source_dir, "pylatex"))

        if "+" not in version and pylatex_exists:
            # This is an official release, just use the pre existing existing
            # python2_source dir
            return

        try:
            # Check if 3to2 exists
            subprocess.check_output(["3to2", "--help"])
            subprocess.check_output(["pasteurize", "--help"])
        except OSError as e:
            if e.errno != errno.ENOENT:
                raise
            if not pylatex_exists:
                raise ImportError(
                    "3to2 and future need to be installed "
                    "before installing when PyLaTeX for Python "
                    "2.7 when it is not installed using one of "
                    "the pip releases."
                )
        else:
            converter = (
                os.path.dirname(os.path.realpath(__file__)) + "/convert_to_py2.sh"
            )
            subprocess.check_call([converter])
            PY2_CONVERTED = True


cmdclass["install"] = CustomInstall
cmdclass["egg_info"] = CustomEggInfo

setup(
    name="PyLaTeX",
    version=version,
    author="Jelte Fennema",
    author_email="pylatex@jeltef.nl",
    description="A Python library for creating LaTeX files and snippets",
    long_description=open("README.rst").read(),
    package_dir={"": source_dir},
    packages=["pylatex", "pylatex.base_classes"],
    url="https://github.com/JelteF/PyLaTeX",
    license="MIT",
    install_requires=dependencies,
    extras_require=extras,
    cmdclass=cmdclass,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Topic :: Software Development :: Code Generators",
        "Topic :: Text Processing :: Markup :: LaTeX",
    ],
)



#!/usr/bin/python
"""
This example shows the functionality of the MiniPage element.

It creates a sample page filled with labels using the MiniPage element.

..  :copyright: (c) 2016 by Vladimir Gorovikov
    :license: MIT, see License for more details.
"""

# begin-doc-include
from pylatex import Document, LineBreak, MiniPage, VerticalSpace


def generate_labels():
    geometry_options = {"margin": "0.5in"}
    doc = Document(geometry_options=geometry_options)

    doc.change_document_style("empty")

    for i in range(10):
        with doc.create(MiniPage(width=r"0.5\textwidth")):
            doc.append("Vladimir Gorovikov")
            doc.append("\n")
            doc.append("Company Name")
            doc.append("\n")
            doc.append("Somewhere, City")
            doc.append("\n")
            doc.append("Country")

        if (i % 2) == 1:
            doc.append(VerticalSpace("20pt"))
            doc.append(LineBreak())

    doc.generate_pdf("minipage", clean_tex=False)


generate_labels()



#!/usr/bin/python
"""
This example shows subfigure functionality.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

# begin-doc-include
import os

from pylatex import Document, Figure, NoEscape, Section, SubFigure

if __name__ == "__main__":
    doc = Document(default_filepath="subfigures")
    image_filename = os.path.join(os.path.dirname(__file__), "kitten.jpg")

    with doc.create(Section("Showing subfigures")):
        with doc.create(Figure(position="h!")) as kittens:
            with doc.create(
                SubFigure(position="b", width=NoEscape(r"0.45\linewidth"))
            ) as left_kitten:
                left_kitten.add_image(image_filename, width=NoEscape(r"\linewidth"))
                left_kitten.add_caption("Kitten on the left")
            with doc.create(
                SubFigure(position="b", width=NoEscape(r"0.45\linewidth"))
            ) as right_kitten:
                right_kitten.add_image(image_filename, width=NoEscape(r"\linewidth"))
                right_kitten.add_caption("Kitten on the right")
            kittens.add_caption("Two kittens")

    doc.generate_pdf(clean_tex=False)



# -*- coding: utf-8 -*-
"""
This module implements LaTeX base classes that can be subclassed.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

from collections import UserList
from contextlib import contextmanager

from pylatex.utils import dumps_list

from .command import Arguments, Command
from .latex_object import LatexObject


class Container(LatexObject, UserList):
    """A base class that groups multiple LaTeX classes.

    This class should be subclassed when a LaTeX class has content that is of
    variable length. It subclasses UserList, so it holds a list of elements
    that can simply be accessed by using normal list functionality, like
    indexing or appending.

    """

    content_separator = "%\n"

    def __init__(self, *, data=None):
        r"""
        Args
        ----
        data: list, `~.LatexObject` or something that can be converted to a \
                string
            The content with which the container is initialized
        """

        if data is None:
            data = []
        elif not isinstance(data, list):
            # If the data is not already a list make it a list, otherwise list
            # operations will not work
            data = [data]

        self.data = data
        self.real_data = data  # Always the data of this instance

        super().__init__()

    @property
    def _repr_attributes(self):
        return super()._repr_attributes + ["real_data"]

    def dumps_content(self, **kwargs):
        r"""Represent the container as a string in LaTeX syntax.

        Args
        ----
        \*\*kwargs:
            Arguments that can be passed to `~.dumps_list`


        Returns
        -------
        string:
            A LaTeX string representing the container
        """

        return dumps_list(
            self, escape=self.escape, token=self.content_separator, **kwargs
        )

    def _propagate_packages(self):
        """Make sure packages get propagated."""

        for item in self.data:
            if isinstance(item, LatexObject):
                if isinstance(item, Container):
                    item._propagate_packages()
                for p in item.packages:
                    self.packages.add(p)

    def dumps_packages(self):
        r"""Represent the packages needed as a string in LaTeX syntax.

        Returns
        -------
        string:
            A LaTeX string representing the packages of the container
        """

        self._propagate_packages()

        return super().dumps_packages()

    @contextmanager
    def create(self, child):
        """Add a LaTeX object to current container, context-manager style.

        Args
        ----
        child: `~.Container`
            An object to be added to the current container
        """

        prev_data = self.data
        self.data = child.data  # This way append works appends to the child

        yield child  # allows with ... as to be used as well

        self.data = prev_data
        self.append(child)


class Environment(Container):
    r"""A base class for LaTeX environments.

    This class implements the basics of a LaTeX environment. A LaTeX
    environment looks like this:

    .. code-block:: latex

        \begin{environment_name}
            Some content that is in the environment
        \end{environment_name}

    The text that is used in the place of environment_name is by default the
    name of the class in lowercase.
    However, this default can be overridden in 2 ways:
    1. setting the _latex_name class variable when declaring the class
    2. setting the _latex_name attribute when initialising object
    """

    #: Set to true if this full container should be equivalent to an empty
    #: string if it has no content.
    omit_if_empty = False

    def __init__(self, *, options=None, arguments=None, start_arguments=None, **kwargs):
        r"""
        Args
        ----
        options: str or list or  `~.Options`
            Options to be added to the ``\begin`` command

        arguments: str or list or `~.Arguments`
            Arguments to be added to the ``\begin`` command

        start_arguments: str or list or `~.Arguments`
            Arguments to be added before the options
        """

        self.options = options
        self.arguments = arguments
        self.start_arguments = start_arguments

        super().__init__(**kwargs)

    def dumps(self):
        """Represent the environment as a string in LaTeX syntax.

        Returns
        -------
        str
            A LaTeX string representing the environment.
        """

        content = self.dumps_content()
        if not content.strip() and self.omit_if_empty:
            return ""

        string = ""

        # Something other than None needs to be used as extra arguments, that
        # way the options end up behind the latex_name argument.
        if self.arguments is None:
            extra_arguments = Arguments()
        else:
            extra_arguments = self.arguments

        begin = Command(
            "begin", self.start_arguments, self.options, extra_arguments=extra_arguments
        )
        begin.arguments._positional_args.insert(0, self.latex_name)
        string += begin.dumps() + self.content_separator

        string += content + self.content_separator

        string += Command("end", self.latex_name).dumps()

        return string


class Fragment(Container):
    r"""A LaTeX fragment container class for fragmented document construction.

    This only provides logical wrapping of the items. The final document will
    look the same as if all items would not have been part of a container.

    A common usecase of this is to generate a .tex snippet containing more than
    one LaTeX item item without any extra container around it. This snippet can
    then be included in another ``.tex`` file using ``\input{snippet.tex}``
    """

    def __init__(self, **kwargs):
        """
        Args
        ----
        """

        super().__init__(**kwargs)

    def dumps(self):
        """Represent the fragment as a string in LaTeX syntax.
        Returns
        -------
        str
        """

        return self.dumps_content()


class ContainerCommand(Container):
    r"""A base class for a container command (A command which contains data).

    Container command example:

    .. code-block:: latex

        \CommandName[options]{arguments}{
            data
        }

    """

    omit_if_empty = False

    def __init__(self, arguments=None, options=None, *, data=None, **kwargs):
        r"""
        Args
        ----
        arguments: str or `list`
            The arguments for the container command
        options: str, list or `~.Options`
            The options for the preamble command
        data: str or `~.LatexObject`
            The data to place inside the preamble command
        """

        self.arguments = arguments
        self.options = options

        super().__init__(data=data, **kwargs)

    def dumps(self):
        r"""Convert the container to a string in latex syntax."""

        content = self.dumps_content()

        if not content.strip() and self.omit_if_empty:
            return ""

        string = ""

        start = Command(self.latex_name, arguments=self.arguments, options=self.options)

        string += start.dumps() + "{%\n"

        if content != "":
            string += content + "%\n}"
        else:
            string += "}"

        return string



# -*- coding: utf-8 -*-
"""
This module implements Error classes.

..  :copyright: (c) 2015 by Rene Beckmann.
    :license: MIT, see License for more details.
"""


class PyLaTeXError(Exception):
    """A Base class for all PyLaTeX Exceptions."""


class CompilerError(PyLaTeXError):
    """A Base class for all PyLaTeX compiler related Exceptions."""


class TableError(PyLaTeXError):
    """A Base class for all errors concerning tables."""


class TableRowSizeError(TableError):
    """Error for wrong table row size."""



#!/usr/bin/python
"""
This example shows basic document generation functionality by inheritance.

..  :copyright: (c) 2017 by Matthias Brandt.
    :license: MIT, see License for more details.
"""

# begin-doc-include
from pylatex import Command, Document, Section, Subsection
from pylatex.utils import NoEscape, italic


class MyDocument(Document):
    def __init__(self):
        super().__init__()

        self.preamble.append(Command("title", "Awesome Title"))
        self.preamble.append(Command("author", "Anonymous author"))
        self.preamble.append(Command("date", NoEscape(r"\today")))
        self.append(NoEscape(r"\maketitle"))

    def fill_document(self):
        """Add a section, a subsection and some text to the document."""
        with self.create(Section("A section")):
            self.append("Some regular text and some ")
            self.append(italic("italic text. "))

            with self.create(Subsection("A subsection")):
                self.append("Also some crazy characters: $&#{}")


if __name__ == "__main__":
    # Document
    doc = MyDocument()

    # Call function to add text
    doc.fill_document()

    # Add stuff to the document
    with doc.create(Section("A second section")):
        doc.append("Some text.")

    doc.generate_pdf("basic_inheritance", clean_tex=False)
    tex = doc.dumps()  # The document as string in LaTeX syntax



# -*- coding: utf-8 -*-
"""
This module implements some simple utility functions.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

import os.path
import shutil
import tempfile

import pylatex.base_classes

_latex_special_chars = {
    "&": r"\&",
    "%": r"\%",
    "$": r"\$",
    "#": r"\#",
    "_": r"\_",
    "{": r"\{",
    "}": r"\}",
    "~": r"\textasciitilde{}",
    "^": r"\^{}",
    "\\": r"\textbackslash{}",
    "\n": "\\newline%\n",
    "-": r"{-}",
    "\xA0": "~",  # Non-breaking space
    "[": r"{[}",
    "]": r"{]}",
}

_tmp_path = None


def _is_iterable(element):
    return hasattr(element, "__iter__") and not isinstance(element, str)


class NoEscape(str):
    """
    A simple string class that is not escaped.

    When a `.NoEscape` string is added to another `.NoEscape` string it will
    produce a `.NoEscape` string. If it is added to normal string it will
    produce a normal string.

    Args
    ----
    string: str
        The content of the `NoEscape` string.
    """

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, self)

    def __add__(self, right):
        s = super().__add__(right)
        if isinstance(right, NoEscape):
            return NoEscape(s)
        return s


def escape_latex(s):
    r"""Escape characters that are special in latex.

    Args
    ----
    s : `str`, `NoEscape` or anything that can be converted to string
        The string to be escaped. If this is not a string, it will be converted
        to a string using `str`. If it is a `NoEscape` string, it will pass
        through unchanged.

    Returns
    -------
    NoEscape
        The string, with special characters in latex escaped.

    Examples
    --------
    >>> escape_latex("Total cost: $30,000")
    'Total cost: \$30,000'
    >>> escape_latex("Issue #5 occurs in 30% of all cases")
    'Issue \#5 occurs in 30\% of all cases'
    >>> print(escape_latex("Total cost: $30,000"))

    References
    ----------
        * http://tex.stackexchange.com/a/34586/43228
        * http://stackoverflow.com/a/16264094/2570866
    """

    if isinstance(s, NoEscape):
        return s

    return NoEscape("".join(_latex_special_chars.get(c, c) for c in str(s)))


def fix_filename(path):
    r"""Fix filenames for use in LaTeX.

    Latex has problems if there are one or more points in the filename, thus
    'abc.def.jpg' will be changed to '{abc.def}.jpg'

    Windows gets angry about the curly braces that resolve the above issue on
    linux Latex distributions. MikTeX however, has no qualms about multiple
    dots in the filename so the behavior is different for posix vs nt when the
    length of file_parts is greater than two.

    Args
    ----
    filename : str
        The file name to be changed.

    Returns
    -------
    str
        The new filename.

    Examples
    --------
    >>> fix_filename("foo.bar.pdf")
    '{foo.bar}.pdf'
    >>> fix_filename("/etc/local/foo.bar.pdf")
    '/etc/local/{foo.bar}.pdf'
    >>> fix_filename("/etc/local/foo.bar.baz/document.pdf")
    '/etc/local/foo.bar.baz/document.pdf'
    >>> fix_filename("/etc/local/foo.bar.baz/foo~1/document.pdf")
    '\detokenize{/etc/local/foo.bar.baz/foo~1/document.pdf}'
    """

    path_parts = path.split("/" if os.name == "posix" else "\\")
    dir_parts = path_parts[:-1]

    filename = path_parts[-1]
    file_parts = filename.split(".")

    if os.name == "posix" and len(file_parts) > 2:
        filename = "{" + ".".join(file_parts[0:-1]) + "}." + file_parts[-1]

    dir_parts.append(filename)
    fixed_path = "/".join(dir_parts)

    if "~" in fixed_path:
        fixed_path = r"\detokenize{" + fixed_path + "}"

    return fixed_path


def dumps_list(l, *, escape=True, token="%\n", mapper=None, as_content=True):
    r"""Try to generate a LaTeX string of a list that can contain anything.

    Args
    ----
    l : list
        A list of objects to be converted into a single string.
    escape : bool
        Whether to escape special LaTeX characters in converted text.
    token : str
        The token (default is a newline) to separate objects in the list.
    mapper: callable or `list`
        A function, class or a list of functions/classes that should be called
        on all entries of the list after converting them to a string, for
        instance `~.bold` or `~.MediumText`.
    as_content: bool
        Indicates whether the items in the list should be dumped using
        `~.LatexObject.dumps_as_content`

    Returns
    -------
    NoEscape
        A single LaTeX string.

    Examples
    --------
    >>> dumps_list([r"\textbf{Test}", r"\nth{4}"])
    '\\textbf{Test}%\n\\nth{4}'
    >>> print(dumps_list([r"\textbf{Test}", r"\nth{4}"]))
    \textbf{Test}
    \nth{4}
    >>> print(pylatex.utils.dumps_list(["There are", 4, "lights!"]))
    There are
    4
    lights!
    >>> print(dumps_list(["$100%", "True"], escape=True))
    \$100\%
    True
    """
    strings = (
        _latex_item_to_string(i, escape=escape, as_content=as_content) for i in l
    )

    if mapper is not None:
        if not isinstance(mapper, list):
            mapper = [mapper]

        for m in mapper:
            strings = [m(s) for s in strings]
        strings = [_latex_item_to_string(s) for s in strings]

    return NoEscape(token.join(strings))


def _latex_item_to_string(item, *, escape=False, as_content=False):
    """Use the render method when possible, otherwise uses str.

    Args
    ----
    item: object
        An object that needs to be converted to a string
    escape: bool
        Flag that indicates if escaping is needed
    as_content: bool
        Indicates whether the item should be dumped using
        `~.LatexObject.dumps_as_content`

    Returns
    -------
    NoEscape
        Latex
    """

    if isinstance(item, pylatex.base_classes.LatexObject):
        if as_content:
            return item.dumps_as_content()
        else:
            return item.dumps()
    elif not isinstance(item, str):
        item = str(item)

    if escape:
        item = escape_latex(item)

    return item


def bold(s, *, escape=True):
    r"""Make a string appear bold in LaTeX formatting.

    bold() wraps a given string in the LaTeX command \textbf{}.

    Args
    ----
    s : str
        The string to be formatted.
    escape: bool
        If true the bold text will be escaped

    Returns
    -------
    NoEscape
        The formatted string.

    Examples
    --------
    >>> bold("hello")
    '\\textbf{hello}'
    >>> print(bold("hello"))
    \textbf{hello}
    """

    if escape:
        s = escape_latex(s)

    return NoEscape(r"\textbf{" + s + "}")


def italic(s, *, escape=True):
    r"""Make a string appear italicized in LaTeX formatting.

    italic() wraps a given string in the LaTeX command \textit{}.

    Args
    ----
    s : str
        The string to be formatted.
    escape: bool
        If true the italic text will be escaped

    Returns
    -------
    NoEscape
        The formatted string.

    Examples
    --------
    >>> italic("hello")
    '\\textit{hello}'
    >>> print(italic("hello"))
    \textit{hello}
    """
    if escape:
        s = escape_latex(s)

    return NoEscape(r"\textit{" + s + "}")


def verbatim(s, *, delimiter="|"):
    r"""Make the string verbatim.

    Wraps the given string in a \verb LaTeX command.

    Args
    ----
    s : str
        The string to be formatted.
    delimiter : str
        How to designate the verbatim text (default is a pipe | )

    Returns
    -------
    NoEscape
        The formatted string.

    Examples
    --------
    >>> verbatim(r"\renewcommand{}")
    '\\verb|\\renewcommand{}|'
    >>> print(verbatim(r"\renewcommand{}"))
    \verb|\renewcommand{}|
    >>> print(verbatim('pi|pe', '!'))
    \verb!pi|pe!
    """

    return NoEscape(r"\verb" + delimiter + s + delimiter)


def make_temp_dir():
    """Create a temporary directory if it doesn't exist.

    Returns
    -------
    str
        The absolute filepath to the created temporary directory.

    Examples
    --------
    >>> make_temp_dir()
    '/var/folders/g9/ct5f3_r52c37rbls5_9nc_qc0000gn/T/pylatex'
    """

    global _tmp_path
    if not _tmp_path:
        _tmp_path = tempfile.mkdtemp(prefix="pylatex-tmp.")
    return _tmp_path


def rm_temp_dir():
    """Remove the temporary directory specified in ``_tmp_path``."""

    global _tmp_path
    if _tmp_path:
        shutil.rmtree(_tmp_path)
        _tmp_path = None



# -*- coding: utf-8 -*-
"""
This module implements the classes that deal with floating environments.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

from . import Command, Environment


class Float(Environment):
    """A class that represents a floating environment."""

    #: By default floats are positioned inside a separate paragraph.
    #: Setting this to option to `False` will change that.
    separate_paragraph = True

    _repr_attributes_mapping = {
        "position": "options",
    }

    def __init__(self, *, position=None, **kwargs):
        """
        Args
        ----
        position: str
            Define the positioning of a floating environment, for instance
            ``'h'``. See the references for more information.

        References
        ----------
            * https://www.sharelatex.com/learn/Positioning_of_Figures
        """

        super().__init__(options=position, **kwargs)

    def add_caption(self, caption):
        """Add a caption to the float.

        Args
        ----
        caption: str
            The text of the caption.
        """

        self.append(Command("caption", caption))



#!/usr/bin/python
"""
Wrapping existing LaTeX environments with the Environment class.

..  :copyright: (c) 2014-2016 by Jelte Fennema, Scott Wallace
    :license: MIT, see License for more details.
"""

# begin-doc-include
from pylatex import Document, Section
from pylatex.base_classes import Environment
from pylatex.package import Package
from pylatex.utils import NoEscape


class AllTT(Environment):
    """A class to wrap LaTeX's alltt environment."""

    packages = [Package("alltt")]
    escape = False
    content_separator = "\n"


# Create a new document
doc = Document()
with doc.create(Section("Wrapping Latex Environments")):
    doc.append(
        NoEscape(
            r"""
        The following is a demonstration of a custom \LaTeX{}
        command with a couple of parameters.
        """
        )
    )

    # Put some data inside the AllTT environment
    with doc.create(AllTT()):
        verbatim = (
            "This is verbatim, alltt, text.\n\n\n"
            "Setting \\underline{escape} to \\underline{False} "
            "ensures that text in the environment is not\n"
            "subject to escaping...\n\n\n"
            "Setting \\underline{content_separator} "
            "ensures that line endings are broken in\n"
            "the latex just as they are in the input text.\n"
            "alltt supports math: \\(x^2=10\\)"
        )
        doc.append(verbatim)

    doc.append("This is back to normal text...")

# Generate pdf
doc.generate_pdf("environment_ex", clean_tex=False)
